import os
import sys
from typing import Optional

from .core import get_registry


class ConfigLoader:
    def __init__(self, config_path: Optional[str] = None):
        self.config_path = config_path
        self._loaded_path: Optional[str] = None

    def find_config_file(self) -> Optional[str]:
        """Find allgood.py configuration file in standard locations."""
        if self.config_path:
            if os.path.exists(self.config_path):
                return os.path.abspath(self.config_path)
            raise FileNotFoundError(f"Config file not found: {self.config_path}")

        # Standard locations to check
        possible_paths = [
            "allgood.py",
            "config/allgood.py",
            os.path.join(os.getcwd(), "allgood.py"),
            os.path.join(os.getcwd(), "config", "allgood.py"),
        ]

        for path in possible_paths:
            if os.path.exists(path):
                return os.path.abspath(path)

        return None

    def load_config(self, environment: str = "development") -> bool:
        """Load configuration file and execute it to register checks."""
        try:
            config_file = self.find_config_file()
            if not config_file:
                return False
        except FileNotFoundError:
            return False

        # Clear existing checks if reloading
        if self._loaded_path != config_file:
            get_registry().clear()

        try:
            # Add the config file's directory to Python path temporarily
            config_dir = os.path.dirname(config_file)
            if config_dir not in sys.path:
                sys.path.insert(0, config_dir)
                path_added = True
            else:
                path_added = False

            # Import DSL functions locally to avoid circular imports
            from .core import check, expect, make_sure

            # Create a namespace with our DSL functions
            namespace = {
                '__file__': config_file,
                '__name__': '__main__',
                'check': check,
                'expect': expect,
                'make_sure': make_sure,
                'ENVIRONMENT': environment,
            }

            # Execute the config file
            with open(config_file) as f:
                code = compile(f.read(), config_file, 'exec')
                exec(code, namespace)

            # Remove from path if we added it
            if path_added:
                sys.path.remove(config_dir)

            self._loaded_path = config_file
            return True

        except Exception as e:
            print(f"Error loading config file {config_file}: {e}")
            return False

    @property
    def loaded_path(self) -> Optional[str]:
        """Return the path of the currently loaded config file."""
        return self._loaded_path


# Import these here to avoid circular imports


def load_config(config_path: Optional[str] = None, environment: str = "development") -> bool:
    """Convenience function to load configuration."""
    loader = ConfigLoader(config_path)
    return loader.load_config(environment)


def find_config() -> Optional[str]:
    """Convenience function to find config file path."""
    loader = ConfigLoader()
    return loader.find_config_file()



from .config import ConfigLoader, find_config, load_config
from .core import (
    AllgreenError,
    Check,
    CheckAssertionError,
    CheckRegistry,
    CheckResult,
    CheckStatus,
    CheckTimeoutError,
    check,
    expect,
    get_registry,
    make_sure,
)

__version__ = "0.1.0"

# Core exports (always available)
__all__ = [
    "check",
    "expect",
    "make_sure",
    "get_registry",
    "CheckRegistry",
    "Check",
    "CheckResult",
    "CheckStatus",
    "AllgreenError",
    "CheckAssertionError",
    "CheckTimeoutError",
    "load_config",
    "find_config",
    "ConfigLoader",
]

# Try to import web framework integrations (optional)
try:
    from .integrations.flask_integration import (  # noqa: F401
        HealthCheckApp,
        create_app,
        mount_healthcheck,
        run_standalone,
    )
    __all__.extend([
        "create_app",
        "mount_healthcheck",
        "run_standalone",
        "HealthCheckApp",
    ])
except ImportError:
    pass

try:
    from .integrations import django_integration  # noqa: F401
    __all__.append("django_integration")
except ImportError:
    pass

try:
    from .integrations import fastapi_integration  # noqa: F401
    __all__.append("fastapi_integration")
except ImportError:
    pass



import tempfile
from datetime import datetime, timedelta
from pathlib import Path

import pytest

from allgreen.rate_limiting import RateLimitConfig, RateLimitTracker


class TestRateLimitConfig:
    def test_parse_valid_patterns(self):
        # Test various valid patterns
        patterns = [
            ("2 times per day", 2, "day"),
            ("4 times per hour", 4, "hour"),
            ("1 time per minute", 1, "minute"),
            ("10 times per day", 10, "day"),
        ]

        for pattern, expected_count, expected_period in patterns:
            config = RateLimitConfig(pattern)
            assert config.count == expected_count
            assert config.period == expected_period

    def test_parse_invalid_patterns(self):
        invalid_patterns = [
            "invalid pattern",
            "2 times per week",  # week not supported
            "abc times per day",
            "2 per day",  # missing "times"
            "",
        ]

        for pattern in invalid_patterns:
            with pytest.raises(ValueError):
                RateLimitConfig(pattern)

    def test_period_durations(self):
        day_config = RateLimitConfig("1 time per day")
        assert day_config.get_period_duration() == timedelta(days=1)

        hour_config = RateLimitConfig("1 time per hour")
        assert hour_config.get_period_duration() == timedelta(hours=1)

        minute_config = RateLimitConfig("1 time per minute")
        assert minute_config.get_period_duration() == timedelta(minutes=1)

    def test_period_start_calculation(self):
        test_time = datetime(2023, 10, 15, 14, 30, 45)  # Oct 15, 2:30:45 PM

        day_config = RateLimitConfig("1 time per day")
        expected_day_start = datetime(2023, 10, 15, 0, 0, 0)
        assert day_config.get_period_start(test_time) == expected_day_start

        hour_config = RateLimitConfig("1 time per hour")
        expected_hour_start = datetime(2023, 10, 15, 14, 0, 0)
        assert hour_config.get_period_start(test_time) == expected_hour_start

        minute_config = RateLimitConfig("1 time per minute")
        expected_minute_start = datetime(2023, 10, 15, 14, 30, 0)
        assert minute_config.get_period_start(test_time) == expected_minute_start


class TestRateLimitTracker:
    def test_basic_rate_limiting(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tracker = RateLimitTracker(Path(tmpdir))
            config = RateLimitConfig("2 times per hour")
            check_id = "test_check"

            # First run should be allowed
            should_run, skip_reason, cached_result = tracker.should_run_check(check_id, config)
            assert should_run is True
            assert skip_reason is None
            assert cached_result is None

            # Second run should be allowed
            should_run, skip_reason, cached_result = tracker.should_run_check(check_id, config)
            assert should_run is True
            assert skip_reason is None

            # Third run should be rate limited
            should_run, skip_reason, cached_result = tracker.should_run_check(check_id, config)
            assert should_run is False
            assert "Rate limited" in skip_reason
            assert "2/2 runs used" in skip_reason

    def test_period_reset(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tracker = RateLimitTracker(Path(tmpdir))
            config = RateLimitConfig("1 time per minute")
            check_id = "test_check"

            now = datetime(2023, 10, 15, 14, 30, 0)  # Minute start

            # Use up the limit
            should_run, _, _ = tracker.should_run_check(check_id, config, now)
            assert should_run is True

            # Should be rate limited in same minute
            should_run, skip_reason, _ = tracker.should_run_check(check_id, config, now)
            assert should_run is False
            assert "Rate limited" in skip_reason

            # Move to next minute - should be allowed again
            next_minute = now + timedelta(minutes=1)
            should_run, _, _ = tracker.should_run_check(check_id, config, next_minute)
            assert should_run is True

    def test_result_caching(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tracker = RateLimitTracker(Path(tmpdir))
            config = RateLimitConfig("1 time per hour")
            check_id = "test_check"

            # Use up the rate limit first
            should_run, _, _ = tracker.should_run_check(check_id, config)
            assert should_run is True

            # Record a result after the first run
            result = {
                "status": "passed",
                "message": "Test passed",
                "duration_ms": 100.0
            }
            tracker.record_result(check_id, result)

            # Should get cached result when rate limited
            should_run, skip_reason, cached_result = tracker.should_run_check(check_id, config)
            assert should_run is False
            assert cached_result == result

    def test_remaining_runs(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tracker = RateLimitTracker(Path(tmpdir))
            config = RateLimitConfig("3 times per hour")
            check_id = "test_check"

            now = datetime(2023, 10, 15, 14, 0, 0)

            # Initially should have 3 runs remaining
            remaining, reset_time = tracker.get_remaining_runs(check_id, config, now)
            assert remaining == 3
            assert reset_time == datetime(2023, 10, 15, 15, 0, 0)  # Next hour

            # Use one run
            tracker.should_run_check(check_id, config, now)
            remaining, _ = tracker.get_remaining_runs(check_id, config, now)
            assert remaining == 2

            # Use another run
            tracker.should_run_check(check_id, config, now)
            remaining, _ = tracker.get_remaining_runs(check_id, config, now)
            assert remaining == 1

    def test_persistence(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir)
            config = RateLimitConfig("2 times per hour")
            check_id = "persistent_test"

            # Create tracker and use up limit
            tracker1 = RateLimitTracker(cache_dir)
            tracker1.should_run_check(check_id, config)
            tracker1.should_run_check(check_id, config)

            # Create new tracker - should remember previous state
            tracker2 = RateLimitTracker(cache_dir)
            should_run, skip_reason, _ = tracker2.should_run_check(check_id, config)
            assert should_run is False
            assert "Rate limited" in skip_reason

    def test_corrupted_cache_handling(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tracker = RateLimitTracker(Path(tmpdir))
            config = RateLimitConfig("1 time per hour")
            check_id = "corrupted_test"

            # Create a corrupted cache file
            cache_file = tracker._get_cache_file(check_id)
            cache_file.parent.mkdir(parents=True, exist_ok=True)
            cache_file.write_text("invalid pickle data")

            # Should handle corruption gracefully
            should_run, _, _ = tracker.should_run_check(check_id, config)
            assert should_run is True  # Should start fresh



import os
import tempfile

from allgreen import get_registry
from allgreen.integrations.flask_integration import create_app


def test_healthcheck_html_endpoint():
    # Clear registry and add a simple check
    registry = get_registry()
    registry.clear()

    # Create a temp config file
    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
        f.write('''
@check("Test passing check")
def test_check():
    make_sure(True)

@check("Test failing check")
def fail_check():
    make_sure(False, "This should fail")
''')
        config_path = f.name

    try:
        # Create Flask app
        app = create_app(
            app_name="Test App",
            config_path=config_path,
            environment="test",
            auto_reload_config=False
        )

        client = app.test_client()

        # Test HTML endpoint
        response = client.get('/healthcheck')
        assert response.status_code == 503  # Should fail due to failing check
        assert b'Test passing check' in response.data
        assert b'Test failing check' in response.data
        assert b'Health Check' in response.data

    finally:
        os.unlink(config_path)


def test_healthcheck_json_endpoint():
    registry = get_registry()
    registry.clear()

    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
        f.write('''
@check("JSON test check")
def json_check():
    expect(2 + 2).to_eq(4)
''')
        config_path = f.name

    try:
        app = create_app(
            app_name="JSON Test App",
            config_path=config_path,
            environment="test"
        )

        client = app.test_client()

        # Test JSON endpoint
        response = client.get('/healthcheck.json')
        assert response.status_code == 200
        assert response.content_type == 'application/json'

        data = response.get_json()
        assert data['status'] == 'passed'
        assert data['app_name'] == 'JSON Test App'
        assert data['environment'] == 'test'
        assert len(data['checks']) == 1
        assert data['checks'][0]['description'] == 'JSON test check'
        assert data['checks'][0]['status'] == 'passed'

    finally:
        os.unlink(config_path)


def test_healthcheck_json_via_accept_header():
    registry = get_registry()
    registry.clear()

    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
        f.write('''
@check("Accept header test")
def header_test():
    make_sure(True)
''')
        config_path = f.name

    try:
        app = create_app(config_path=config_path)
        client = app.test_client()

        # Test with Accept: application/json header
        response = client.get(
            '/healthcheck',
            headers={'Accept': 'application/json'}
        )
        assert response.status_code == 200
        assert response.content_type == 'application/json'

        data = response.get_json()
        assert data['status'] == 'passed'

    finally:
        os.unlink(config_path)


def test_healthcheck_format_parameter():
    registry = get_registry()
    registry.clear()

    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
        f.write('''
@check("Format param test")
def format_test():
    make_sure(True)
''')
        config_path = f.name

    try:
        app = create_app(config_path=config_path)
        client = app.test_client()

        # Test with ?format=json parameter
        response = client.get('/healthcheck?format=json')
        assert response.status_code == 200
        assert response.content_type == 'application/json'

    finally:
        os.unlink(config_path)


def test_healthcheck_statistics():
    registry = get_registry()
    registry.clear()

    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
        f.write('''
@check("Passing check")
def pass_check():
    make_sure(True)

@check("Failing check")
def fail_check():
    make_sure(False)

@check("Skipped check", only="production")
def skip_check():
    make_sure(True)
''')
        config_path = f.name

    try:
        app = create_app(
            config_path=config_path,
            environment="development"  # Will skip the production-only check
        )
        client = app.test_client()

        response = client.get('/healthcheck.json')
        data = response.get_json()

        assert data['stats']['total'] == 3
        assert data['stats']['passed'] == 1
        assert data['stats']['failed'] == 1
        assert data['stats']['skipped'] == 1
        assert data['status'] == 'failed'  # Overall status should be failed

    finally:
        os.unlink(config_path)



import os
import tempfile

from allgreen import find_config, get_registry, load_config


def test_find_config_file():
    # Test with a temporary config file
    with tempfile.TemporaryDirectory() as tmpdir:
        config_path = os.path.join(tmpdir, "allgood.py")
        with open(config_path, 'w') as f:
            f.write('# test config')

        # Change to temp directory
        original_cwd = os.getcwd()
        try:
            os.chdir(tmpdir)
            found_path = find_config()
            assert found_path is not None
            assert found_path.endswith("allgood.py")
        finally:
            os.chdir(original_cwd)


def test_load_config_file():
    # Clear registry first
    registry = get_registry()
    registry.clear()

    # Create a temporary config file
    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
        f.write('''
@check("Test config check")
def test_check():
    make_sure(True)

@check("Math check from config")
def math_check():
    expect(5).to_be_greater_than(3)
''')
        config_path = f.name

    try:
        # Load the config
        success = load_config(config_path)
        assert success, "Config should load successfully"

        # Check that checks were registered
        checks = registry.get_checks()
        assert len(checks) == 2

        descriptions = [check.description for check in checks]
        assert "Test config check" in descriptions
        assert "Math check from config" in descriptions

        # Run the checks to make sure they work
        results = registry.run_all()
        assert len(results) == 2
        assert all(result.passed for _, result in results)

    finally:
        os.unlink(config_path)


def test_load_nonexistent_config():
    success = load_config("/nonexistent/path/allgood.py")
    assert not success, "Should fail to load nonexistent config"


def test_config_with_environment_conditions():
    registry = get_registry()
    registry.clear()

    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
        f.write('''
@check("Production only", only="production")
def prod_check():
    make_sure(True)

@check("Environment aware check")
def env_check():
    make_sure(ENVIRONMENT in ["development", "production"])
''')
        config_path = f.name

    try:
        # Load with development environment
        success = load_config(config_path, environment="development")
        assert success

        checks = registry.get_checks()
        assert len(checks) == 2

        # Run checks
        results = registry.run_all("development")

        # First check should be skipped (production only)
        assert results[0][1].skipped
        # Second check should pass
        assert results[1][1].passed

    finally:
        os.unlink(config_path)



"""
Rate limiting functionality for expensive health checks.

Supports patterns like:
- "2 times per day"
- "4 times per hour"
- "1 time per minute"
"""

import pickle
import re
import threading
from datetime import datetime, timedelta
from pathlib import Path
from typing import Dict, Optional, Tuple

DEFAULT_CACHE_DIR = Path.home() / ".allgreen" / "rate_limits"


class RateLimitConfig:
    """Configuration for a rate-limited check."""

    def __init__(self, run_pattern: str):
        self.pattern = run_pattern
        self.count, self.period = self._parse_pattern(run_pattern)

    def _parse_pattern(self, pattern: str) -> Tuple[int, str]:
        """Parse patterns like '2 times per day' into (2, 'day')."""
        # Normalize the pattern
        pattern = pattern.lower().strip()

        # Match patterns like "2 times per day", "4 times per hour", etc.
        match = re.match(r"(\d+)\s+times?\s+per\s+(day|hour|minute)", pattern)
        if not match:
            raise ValueError(f"Invalid rate limit pattern: {pattern}")

        count = int(match.group(1))
        period = match.group(2)

        return count, period

    def get_period_duration(self) -> timedelta:
        """Get the duration of one period."""
        if self.period == "day":
            return timedelta(days=1)
        elif self.period == "hour":
            return timedelta(hours=1)
        elif self.period == "minute":
            return timedelta(minutes=1)
        else:
            raise ValueError(f"Unsupported period: {self.period}")

    def get_period_start(self, now: Optional[datetime] = None) -> datetime:
        """Get the start of the current period."""
        if now is None:
            now = datetime.now()

        if self.period == "day":
            # Start of day
            return now.replace(hour=0, minute=0, second=0, microsecond=0)
        elif self.period == "hour":
            # Start of hour
            return now.replace(minute=0, second=0, microsecond=0)
        elif self.period == "minute":
            # Start of minute
            return now.replace(second=0, microsecond=0)
        else:
            raise ValueError(f"Unsupported period: {self.period}")


class RateLimitTracker:
    """Thread-safe rate limit tracking with persistent storage."""

    def __init__(self, cache_dir: Optional[Path] = None):
        self.cache_dir = cache_dir or DEFAULT_CACHE_DIR
        self.cache_dir.mkdir(parents=True, exist_ok=True)
        self._lock = threading.Lock()

    def _get_cache_file(self, check_id: str) -> Path:
        """Get the cache file path for a specific check."""
        # Use check description as ID, but sanitize for filesystem
        safe_id = re.sub(r'[^\w\-_.]', '_', check_id)
        return self.cache_dir / f"{safe_id}.pkl"

    def _load_state(self, check_id: str) -> Dict:
        """Load the rate limit state from disk."""
        cache_file = self._get_cache_file(check_id)
        if not cache_file.exists():
            return {"count": 0, "period_start": None, "last_result": None}

        try:
            with open(cache_file, "rb") as f:
                return pickle.load(f)
        except (OSError, pickle.PickleError):
            # If cache is corrupted, start fresh
            return {"count": 0, "period_start": None, "last_result": None}

    def _save_state(self, check_id: str, state: Dict) -> None:
        """Save the rate limit state to disk."""
        cache_file = self._get_cache_file(check_id)
        try:
            with open(cache_file, "wb") as f:
                pickle.dump(state, f)
        except OSError:
            # If we can't save to cache, continue without persistence
            pass

    def should_run_check(
        self, check_id: str, config: RateLimitConfig, now: Optional[datetime] = None
    ) -> Tuple[bool, Optional[str], Optional[dict]]:
        """
        Check if a rate-limited check should run.

        Returns:
            (should_run, skip_reason, cached_result)
        """
        if now is None:
            now = datetime.now()

        with self._lock:
            state = self._load_state(check_id)

            period_start = config.get_period_start(now)
            current_period_start = state.get("period_start")

            # If we're in a new period, reset the counter
            if (
                current_period_start is None
                or period_start > current_period_start
            ):
                state = {
                    "count": 0,
                    "period_start": period_start,
                    "last_result": state.get("last_result"),
                }

            # Check if we've exceeded the rate limit
            if state["count"] >= config.count:
                # Calculate when the next period starts
                next_period = period_start + config.get_period_duration()
                remaining_time = next_period - now

                if remaining_time.total_seconds() > 0:
                    # Format remaining time
                    if remaining_time.days > 0:
                        time_str = f"{remaining_time.days}d {remaining_time.seconds // 3600}h"
                    elif remaining_time.seconds > 3600:
                        time_str = f"{remaining_time.seconds // 3600}h {(remaining_time.seconds % 3600) // 60}m"
                    else:
                        time_str = f"{remaining_time.seconds // 60}m"

                    skip_reason = (
                        f"Rate limited: {state['count']}/{config.count} runs used this {config.period}. "
                        f"Next run in {time_str}"
                    )
                    return False, skip_reason, state.get("last_result")

            # Should run - increment counter
            state["count"] += 1
            self._save_state(check_id, state)

            return True, None, None

    def record_result(
        self, check_id: str, result: dict, now: Optional[datetime] = None
    ) -> None:
        """Record the result of a rate-limited check."""
        with self._lock:
            state = self._load_state(check_id)
            state["last_result"] = result
            self._save_state(check_id, state)

    def get_remaining_runs(
        self, check_id: str, config: RateLimitConfig, now: Optional[datetime] = None
    ) -> Tuple[int, datetime]:
        """Get the number of remaining runs and when the limit resets."""
        if now is None:
            now = datetime.now()

        with self._lock:
            state = self._load_state(check_id)
            period_start = config.get_period_start(now)

            # If we're in a new period, all runs are available
            if (
                state.get("period_start") is None
                or period_start > state.get("period_start")
            ):
                remaining = config.count
            else:
                remaining = max(0, config.count - state.get("count", 0))

            next_reset = period_start + config.get_period_duration()
            return remaining, next_reset


# Global rate limit tracker instance
_rate_tracker = RateLimitTracker()


def get_rate_tracker() -> RateLimitTracker:
    """Get the global rate limit tracker instance."""
    return _rate_tracker


def set_rate_tracker_cache_dir(cache_dir: Path) -> None:
    """Set the cache directory for rate limiting data."""
    global _rate_tracker
    _rate_tracker = RateLimitTracker(cache_dir)



import os
from datetime import datetime
from typing import Any, Dict, List, Optional, Tuple

from flask import Flask, Response, jsonify, render_template, request

import allgreen

from ..config import load_config
from ..core import Check, CheckResult, CheckStatus, get_registry


class HealthCheckApp:
    def __init__(
        self,
        app_name: str = "Application",
        config_path: Optional[str] = None,
        environment: Optional[str] = None,
        auto_reload_config: bool = True
    ):
        self.app_name = app_name
        self.config_path = config_path
        self.environment = environment or os.getenv("ENVIRONMENT", "development")
        self.auto_reload_config = auto_reload_config
        self._last_config_mtime = None

        # Load initial config
        if auto_reload_config or not get_registry().get_checks():
            self._load_config()

    def _load_config(self) -> bool:
        """Load or reload configuration if needed."""
        if not self.auto_reload_config:
            return load_config(self.config_path, self.environment)

        # Check if we should reload config based on file modification time
        if self.config_path and os.path.exists(self.config_path):
            current_mtime = os.path.getmtime(self.config_path)
            if self._last_config_mtime is None or current_mtime > self._last_config_mtime:
                self._last_config_mtime = current_mtime
                return load_config(self.config_path, self.environment)
        elif not self.config_path:
            # Try to find and load config file
            return load_config(None, self.environment)

        return True

    def _calculate_stats(self, results: List[Tuple[Check, CheckResult]]) -> Dict[str, int]:
        """Calculate statistics from check results."""
        stats = {
            "total": len(results),
            "passed": 0,
            "failed": 0,
            "skipped": 0,
            "error": 0
        }

        for _, result in results:
            if result.status == CheckStatus.PASSED:
                stats["passed"] += 1
            elif result.status == CheckStatus.FAILED:
                stats["failed"] += 1
            elif result.status == CheckStatus.SKIPPED:
                stats["skipped"] += 1
            elif result.status == CheckStatus.ERROR:
                stats["error"] += 1

        # Combine failed and error for simpler display
        stats["failed"] += stats["error"]

        return stats

    def _get_overall_status(self, stats: Dict[str, int]) -> str:
        """Determine overall health status."""
        if stats["failed"] > 0:
            return "failed"
        elif stats["total"] == stats["skipped"]:
            return "no_checks"
        elif stats["passed"] > 0:
            return "passed"
        else:
            return "unknown"

    def run_health_checks(self) -> Tuple[List[Tuple[Check, CheckResult]], Dict[str, Any]]:
        """Run all health checks and return results with metadata."""
        # Reload config if needed
        self._load_config()

        # Get all checks and run them
        registry = get_registry()
        results = registry.run_all(self.environment)

        # Calculate statistics
        stats = self._calculate_stats(results)
        overall_status = self._get_overall_status(stats)

        metadata = {
            "stats": stats,
            "overall_status": overall_status,
            "environment": self.environment,
            "app_name": self.app_name,
            "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        }

        return results, metadata

    def healthcheck_html(self) -> Tuple[str, int]:
        """Generate HTML health check page."""
        results, metadata = self.run_health_checks()

        # Determine HTTP status code
        status_code = 200 if metadata["overall_status"] == "passed" else 503

        # Render template
        html = render_template(
            "healthcheck.html",
            results=results,
            **metadata
        )

        return html, status_code

    def healthcheck_json(self) -> Tuple[Dict[str, Any], int]:
        """Generate JSON health check response."""
        results, metadata = self.run_health_checks()

        # Convert results to JSON-serializable format
        json_results = []
        for check, result in results:
            json_results.append({
                "description": check.description,
                "status": result.status.value,
                "message": result.message,
                "error": result.error,
                "duration_ms": result.duration_ms,
                "skip_reason": result.skip_reason,
            })

        response_data = {
            "status": metadata["overall_status"],
            "stats": metadata["stats"],
            "environment": metadata["environment"],
            "app_name": metadata["app_name"],
            "timestamp": metadata["timestamp"],
            "checks": json_results,
        }

        # Determine HTTP status code
        status_code = 200 if metadata["overall_status"] == "passed" else 503

        return response_data, status_code


def create_app(
    app_name: str = "Application",
    config_path: Optional[str] = None,
    environment: Optional[str] = None,
    auto_reload_config: bool = True,
    flask_app: Optional[Flask] = None
) -> Flask:
    """Create and configure a Flask app with health check endpoints."""

    if flask_app is None:
        flask_app = Flask(__name__, template_folder=os.path.join(os.path.dirname(allgreen.__file__), 'templates'))

    # Create health check instance
    health_checker = HealthCheckApp(
        app_name=app_name,
        config_path=config_path,
        environment=environment,
        auto_reload_config=auto_reload_config
    )

    @flask_app.route("/healthcheck")
    def healthcheck():
        """Health check endpoint that returns HTML by default, JSON if requested."""
        accept_header = request.headers.get("Accept", "")
        if "application/json" in accept_header or request.args.get("format") == "json":
            data, status_code = health_checker.healthcheck_json()
            return jsonify(data), status_code
        else:
            html, status_code = health_checker.healthcheck_html()
            return Response(html, status=status_code, mimetype="text/html")

    @flask_app.route("/healthcheck.json")
    def healthcheck_json():
        """Explicit JSON health check endpoint."""
        data, status_code = health_checker.healthcheck_json()
        return jsonify(data), status_code

    return flask_app


def mount_healthcheck(
    app: Flask,
    app_name: str = "Application",
    config_path: Optional[str] = None,
    environment: Optional[str] = None,
    auto_reload_config: bool = True,
    url_prefix: str = ""
) -> Flask:
    """Mount health check routes on an existing Flask app."""

    # Create health check app
    health_app = create_app(
        app_name=app_name,
        config_path=config_path,
        environment=environment,
        auto_reload_config=auto_reload_config,
        flask_app=app
    )

    return health_app


# For standalone usage
def run_standalone(
    host: str = "127.0.0.1",
    port: int = 5000,
    debug: bool = True,
    **kwargs
):
    """Run a standalone health check server."""
    app = create_app(**kwargs)
    app.run(host=host, port=port, debug=debug)



"""
Django integration for allgreen health checks.

Usage:
    # In views.py
    from allgreen.integrations import django_integration

    # Add to urlpatterns
    urlpatterns = [
        path('healthcheck/', django_integration.healthcheck_view, name='healthcheck'),
    ]

    # Or use as class-based view
    urlpatterns = [
        path('healthcheck/', django_integration.HealthCheckView.as_view(), name='healthcheck'),
    ]
"""

from datetime import datetime
from typing import Optional

try:
    from django.http import HttpRequest, HttpResponse, JsonResponse
    from django.template.loader import render_to_string
    from django.utils.decorators import method_decorator
    from django.views import View
    from django.views.decorators.cache import never_cache
except ImportError:
    raise ImportError(
        "Django is required for django_integration. "
        "Install with: pip install allgreen[django]"
    ) from None

from ..config import load_config
from ..core import CheckStatus, get_registry


class HealthCheckView(View):
    """
    Django class-based view for health checks.

    Usage:
        urlpatterns = [
            path('healthcheck/', HealthCheckView.as_view(), name='healthcheck'),
        ]
    """

    app_name = "Django Application"
    config_path = None
    environment = None

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)

    def get(self, request: HttpRequest) -> HttpResponse:
        return healthcheck_view(
            request,
            app_name=self.app_name,
            config_path=self.config_path,
            environment=self.environment
        )


def healthcheck_view(
    request: HttpRequest,
    app_name: str = "Django Application",
    config_path: Optional[str] = None,
    environment: Optional[str] = None
) -> HttpResponse:
    """
    Django function-based view for health checks.

    Returns HTML or JSON based on Accept header or ?format parameter.
    HTTP status codes: 200 OK if all pass, 503 Service Unavailable if any fail.

    Args:
        request: Django HTTP request
        app_name: Application name to display
        config_path: Path to allgood.py config file
        environment: Environment name (defaults to 'development')
    """

    # Load configuration and run checks
    if environment is None:
        environment = "development"

    load_config(config_path, environment)
    registry = get_registry()
    results = registry.run_all(environment)

    # Calculate statistics and overall status
    stats = _calculate_stats(results)
    overall_status = _get_overall_status(stats)

    # Determine response format
    wants_json = (
        'application/json' in request.headers.get('Accept', '') or
        request.GET.get('format') == 'json'
    )

    # Determine HTTP status code
    status_code = 200 if overall_status == "passed" else 503

    if wants_json:
        # Return JSON response
        return JsonResponse(
            _format_json_response(results, stats, overall_status, app_name, environment),
            status=status_code
        )
    else:
        # Return HTML response
        context = {
            'results': results,
            'stats': stats,
            'overall_status': overall_status,
            'app_name': app_name,
            'environment': environment,
            'timestamp': datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        }

        html_content = _render_html_template(context)
        return HttpResponse(html_content, status=status_code, content_type='text/html')


def _calculate_stats(results):
    """Calculate statistics from check results."""
    stats = {
        "total": len(results),
        "passed": 0,
        "failed": 0,
        "skipped": 0,
        "error": 0
    }

    for _, result in results:
        if result.status == CheckStatus.PASSED:
            stats["passed"] += 1
        elif result.status == CheckStatus.FAILED:
            stats["failed"] += 1
        elif result.status == CheckStatus.SKIPPED:
            stats["skipped"] += 1
        elif result.status == CheckStatus.ERROR:
            stats["error"] += 1

    # Combine failed and error for simpler display
    stats["failed"] += stats["error"]

    return stats


def _get_overall_status(stats):
    """Determine overall health status."""
    if stats["failed"] > 0:
        return "failed"
    elif stats["total"] == stats["skipped"]:
        return "no_checks"
    elif stats["passed"] > 0:
        return "passed"
    else:
        return "unknown"


def _format_json_response(results, stats, overall_status, app_name, environment):
    """Format results for JSON response."""
    json_results = []
    for check, result in results:
        json_results.append({
            "description": check.description,
            "status": result.status.value,
            "message": result.message,
            "error": result.error,
            "duration_ms": result.duration_ms,
            "skip_reason": result.skip_reason,
        })

    return {
        "status": overall_status,
        "stats": stats,
        "environment": environment,
        "app_name": app_name,
        "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        "checks": json_results,
    }


def _render_html_template(context):
    """
    Render HTML template.

    First tries to use Django template loader to find 'allgreen/healthcheck.html',
    falls back to inline template if not found.
    """
    try:
        # Try to use Django template
        return render_to_string('allgreen/healthcheck.html', context)
    except Exception:
        # Fall back to inline template (same as Flask version)
        return _get_inline_template().format(**context)


def _get_inline_template():
    """Inline HTML template as fallback."""
    # This would contain the same HTML template as the Flask version
    # For brevity, returning a simple template here
    return """
<!DOCTYPE html>
<html>
<head>
    <title>Health Check - {app_name}</title>
    <style>
        body {{ font-family: sans-serif; margin: 40px; }}
        .pass {{ color: green; }}
        .fail {{ color: red; }}
        .skip {{ color: orange; }}
    </style>
</head>
<body>
    <h1>Health Check: {app_name}</h1>
    <p>Status: <strong>{overall_status}</strong></p>
    <p>Environment: {environment}</p>
    <p>Timestamp: {timestamp}</p>

    <h2>Results</h2>
    <ul>
    {% for check, result in results %}
        <li class="{result.status.value}">
            {check.description}: {result.status.value}
            {% if result.skip_reason %} - {result.skip_reason}{% endif %}
            {% if result.message and result.status.value != "passed" %} - {result.message}{% endif %}
        </li>
    {% endfor %}
    </ul>
</body>
</html>
"""



import pytest

from allgreen import (
    CheckAssertionError,
    CheckStatus,
    check,
    expect,
    get_registry,
    make_sure,
)


def test_basic_check_passing():
    # Clear registry for clean test
    registry = get_registry()
    registry.clear()

    @check("Simple passing check")
    def simple_check():
        make_sure(True)

    checks = registry.get_checks()
    assert len(checks) == 1

    check_obj = checks[0]
    result = check_obj.execute()

    assert result.passed
    assert result.status == CheckStatus.PASSED
    assert "Check passed" in result.message


def test_basic_check_failing():
    registry = get_registry()
    registry.clear()

    @check("Simple failing check")
    def failing_check():
        make_sure(False, "This should fail")

    checks = registry.get_checks()
    check_obj = checks[0]
    result = check_obj.execute()

    assert result.failed
    assert result.status == CheckStatus.FAILED
    assert "This should fail" in result.message


def test_expectation_methods():
    # Test to_eq
    expect(5).to_eq(5)
    with pytest.raises(CheckAssertionError):
        expect(5).to_eq(10)

    # Test to_be_greater_than
    expect(10).to_be_greater_than(5)
    with pytest.raises(CheckAssertionError):
        expect(5).to_be_greater_than(10)

    # Test to_be_less_than
    expect(5).to_be_less_than(10)
    with pytest.raises(CheckAssertionError):
        expect(10).to_be_less_than(5)


def test_check_with_expectations():
    registry = get_registry()
    registry.clear()

    @check("Check with expectations")
    def expectation_check():
        expect(2 + 2).to_eq(4)
        expect(10).to_be_greater_than(5)
        expect(3).to_be_less_than(8)

    check_obj = registry.get_checks()[0]
    result = check_obj.execute()

    assert result.passed


def test_environment_conditions():
    registry = get_registry()
    registry.clear()

    @check("Production only check", only="production")
    def prod_check():
        make_sure(True)

    @check("Skip in development", except_env="development")
    def skip_dev_check():
        make_sure(True)

    checks = registry.get_checks()
    prod_result = checks[0].execute("development")
    skip_result = checks[1].execute("development")

    assert prod_result.skipped
    assert skip_result.skipped
    assert "Only runs in production" in prod_result.skip_reason
    assert "Skipped in development" in skip_result.skip_reason


def test_if_condition():
    registry = get_registry()
    registry.clear()

    @check("Conditional check", if_condition=lambda: 1 + 1 == 2)
    def conditional_check():
        make_sure(True)

    @check("False condition check", if_condition=False)
    def false_condition_check():
        make_sure(True)

    checks = registry.get_checks()
    true_result = checks[0].execute()
    false_result = checks[1].execute()

    assert true_result.passed
    assert false_result.skipped
    assert "Custom condition is False" in false_result.skip_reason



import signal
import threading
import time
import traceback
from contextlib import contextmanager
from dataclasses import dataclass
from enum import Enum
from typing import Any, Callable, List, Optional, Union


class CheckStatus(Enum):
    PASSED = "passed"
    FAILED = "failed"
    SKIPPED = "skipped"
    ERROR = "error"


@dataclass
class CheckResult:
    status: CheckStatus
    message: Optional[str] = None
    error: Optional[str] = None
    duration_ms: Optional[float] = None
    skip_reason: Optional[str] = None

    @property
    def passed(self) -> bool:
        return self.status == CheckStatus.PASSED

    @property
    def failed(self) -> bool:
        return self.status in (CheckStatus.FAILED, CheckStatus.ERROR)

    @property
    def skipped(self) -> bool:
        return self.status == CheckStatus.SKIPPED


class AllgreenError(Exception):
    pass


class CheckAssertionError(AllgreenError):
    pass


class CheckTimeoutError(AllgreenError):
    pass


@contextmanager
def timeout_context(seconds: int):
    """Context manager for timing out function execution."""
    # Check if we're in the main thread and signals are available
    is_main_thread = threading.current_thread() is threading.main_thread()

    if hasattr(signal, 'SIGALRM') and is_main_thread:
        # Unix systems in main thread - use signals (more reliable)
        def timeout_handler(signum, frame):
            raise CheckTimeoutError(f"Check timed out after {seconds} seconds")

        old_handler = signal.signal(signal.SIGALRM, timeout_handler)
        signal.alarm(seconds)
        try:
            yield
        finally:
            signal.alarm(0)
            signal.signal(signal.SIGALRM, old_handler)
    else:
        # Threading-based timeout (works in all threads and systems)
        timer_expired = threading.Event()

        def timeout_func():
            timer_expired.set()

        timer = threading.Timer(seconds, timeout_func)
        timer.start()

        try:
            start_time = time.time()
            yield
            # Check if we timed out during execution
            if timer_expired.is_set() or (time.time() - start_time) >= seconds:
                raise CheckTimeoutError(f"Check timed out after {seconds} seconds")
        finally:
            timer.cancel()


class Expectation:
    def __init__(self, actual: Any):
        self.actual = actual

    def to_eq(self, expected: Any) -> None:
        if self.actual != expected:
            raise CheckAssertionError(
                f"Expected {self.actual!r} to equal {expected!r}"
            )

    def to_be_greater_than(self, expected: Union[int, float]) -> None:
        if not (isinstance(self.actual, (int, float)) and self.actual > expected):
            raise CheckAssertionError(
                f"Expected {self.actual!r} to be greater than {expected!r}"
            )

    def to_be_less_than(self, expected: Union[int, float]) -> None:
        if not (isinstance(self.actual, (int, float)) and self.actual < expected):
            raise CheckAssertionError(
                f"Expected {self.actual!r} to be less than {expected!r}"
            )


def expect(actual: Any) -> Expectation:
    return Expectation(actual)


def make_sure(condition: Any, message: Optional[str] = None) -> None:
    if not condition:
        raise CheckAssertionError(message or f"Expected {condition!r} to be truthy")


class Check:
    def __init__(
        self,
        description: str,
        func: Callable[[], None],
        timeout: Optional[int] = None,
        only: Optional[Union[str, List[str]]] = None,
        except_env: Optional[Union[str, List[str]]] = None,
        if_condition: Optional[Union[bool, Callable[[], bool]]] = None,
        run: Optional[str] = None,
    ):
        self.description = description
        self.func = func
        self.timeout = timeout or 10  # Default 10 second timeout
        self.only = self._normalize_env_list(only)
        self.except_env = self._normalize_env_list(except_env)
        self.if_condition = if_condition
        self.run = run

    def _normalize_env_list(self, env: Optional[Union[str, List[str]]]) -> Optional[List[str]]:
        if env is None:
            return None
        if isinstance(env, str):
            return [env]
        return env

    def should_run(self, environment: str = "development") -> tuple[bool, Optional[str]]:
        # Check environment conditions
        if self.only and environment not in self.only:
            return False, f"Only runs in {', '.join(self.only)}, current: {environment}"

        if self.except_env and environment in self.except_env:
            return False, f"Skipped in {environment} environment"

        # Check if condition
        if self.if_condition is not None:
            if callable(self.if_condition):
                try:
                    if not self.if_condition():
                        return False, "Custom condition not met"
                except Exception as e:
                    return False, f"Custom condition failed: {e}"
            elif not self.if_condition:
                return False, "Custom condition is False"

        return True, None

    def execute(self, environment: str = "development") -> CheckResult:
        # Check basic conditions (environment, if_condition)
        should_run, skip_reason = self.should_run(environment)
        if not should_run:
            return CheckResult(
                status=CheckStatus.SKIPPED,
                skip_reason=skip_reason
            )

        # Check rate limiting if specified
        if self.run:
            should_run_rate, skip_reason_rate, cached_result = self._check_rate_limit()
            if not should_run_rate:
                # Return cached result if we have one, otherwise create skipped result
                if cached_result:
                    # For rate-limited checks, we return SKIPPED status but include cached info
                    return CheckResult(
                        status=CheckStatus.SKIPPED,
                        message=f"Cached result: {cached_result.get('message', 'N/A')}",
                        skip_reason=skip_reason_rate
                    )
                else:
                    return CheckResult(
                        status=CheckStatus.SKIPPED,
                        skip_reason=skip_reason_rate
                    )

        start_time = time.time()
        try:
            # Execute with timeout
            with timeout_context(self.timeout):
                self.func()

            duration_ms = (time.time() - start_time) * 1000
            result = CheckResult(
                status=CheckStatus.PASSED,
                message="Check passed",
                duration_ms=duration_ms
            )

            # Cache result for rate-limited checks
            if self.run:
                self._cache_result(result)

            return result

        except CheckTimeoutError as e:
            duration_ms = (time.time() - start_time) * 1000
            result = CheckResult(
                status=CheckStatus.ERROR,
                error=str(e),
                message="Check timed out",
                duration_ms=duration_ms
            )
            if self.run:
                self._cache_result(result)
            return result

        except CheckAssertionError as e:
            duration_ms = (time.time() - start_time) * 1000
            result = CheckResult(
                status=CheckStatus.FAILED,
                message=str(e),
                duration_ms=duration_ms
            )
            if self.run:
                self._cache_result(result)
            return result

        except Exception as e:
            duration_ms = (time.time() - start_time) * 1000
            result = CheckResult(
                status=CheckStatus.ERROR,
                error=f"{type(e).__name__}: {e}",
                message=traceback.format_exc(),
                duration_ms=duration_ms
            )
            if self.run:
                self._cache_result(result)
            return result

    def _check_rate_limit(self) -> tuple[bool, Optional[str], Optional[dict]]:
        """Check if this rate-limited check should run."""
        if not self.run:
            return True, None, None

        # Import locally to avoid circular imports
        from .rate_limiting import RateLimitConfig, get_rate_tracker

        try:
            config = RateLimitConfig(self.run)
            tracker = get_rate_tracker()
            return tracker.should_run_check(self.description, config)
        except ValueError:
            # Invalid rate limit pattern - run the check but log error
            return True, None, None

    def _cache_result(self, result: CheckResult) -> None:
        """Cache the result of a rate-limited check."""
        if not self.run:
            return

        from .rate_limiting import get_rate_tracker

        # Convert CheckResult to dict for caching
        result_dict = {
            "status": result.status,
            "message": result.message,
            "error": result.error,
            "duration_ms": result.duration_ms,
        }

        tracker = get_rate_tracker()
        tracker.record_result(self.description, result_dict)


class CheckRegistry:
    def __init__(self):
        self._checks: List[Check] = []

    def register(self, check: Check) -> None:
        self._checks.append(check)

    def get_checks(self) -> List[Check]:
        return self._checks.copy()

    def clear(self) -> None:
        self._checks.clear()

    def run_all(self, environment: str = "development") -> List[tuple[Check, CheckResult]]:
        results = []
        for check in self._checks:
            result = check.execute(environment)
            results.append((check, result))
        return results


# Global registry
_registry = CheckRegistry()


def check(
    description: str,
    timeout: Optional[int] = None,
    only: Optional[Union[str, List[str]]] = None,
    except_env: Optional[Union[str, List[str]]] = None,
    if_condition: Optional[Union[bool, Callable[[], bool]]] = None,
    run: Optional[str] = None,
):
    def decorator(func: Callable[[], None]) -> Callable[[], None]:
        check_obj = Check(
            description=description,
            func=func,
            timeout=timeout,
            only=only,
            except_env=except_env,
            if_condition=if_condition,
            run=run,
        )
        _registry.register(check_obj)
        return func

    return decorator


def get_registry() -> CheckRegistry:
    return _registry



"""
FastAPI integration for allgreen health checks.

Usage:
    from fastapi import FastAPI
    from allgreen.integrations import fastapi_integration

    app = FastAPI()

    # Method 1: Mount the router
    app.include_router(
        fastapi_integration.create_router(app_name="My FastAPI App"),
        prefix="/health"
    )

    # Method 2: Add individual route
    @app.get("/healthcheck")
    async def health():
        return await fastapi_integration.healthcheck_endpoint()
"""

from datetime import datetime
from typing import Optional

try:
    from fastapi import APIRouter, Request
    from fastapi.responses import HTMLResponse, JSONResponse
    from jinja2 import Template
except ImportError:
    raise ImportError(
        "FastAPI and Jinja2 are required for fastapi_integration. "
        "Install with: pip install allgreen[fastapi]"
    ) from None

from ..config import load_config
from ..core import CheckStatus, get_registry


def create_router(
    app_name: str = "FastAPI Application",
    config_path: Optional[str] = None,
    environment: Optional[str] = None,
    prefix: str = ""
) -> APIRouter:
    """
    Create a FastAPI router with health check endpoints.

    Args:
        app_name: Application name to display
        config_path: Path to allgood.py config file
        environment: Environment name
        prefix: URL prefix for routes

    Returns:
        APIRouter with /healthcheck and /healthcheck.json endpoints
    """
    router = APIRouter()

    @router.get("/healthcheck", response_class=HTMLResponse)
    @router.get("/healthcheck.json", response_class=JSONResponse)
    async def healthcheck_endpoint(request: Request):
        return await _healthcheck_handler(
            request, app_name, config_path, environment
        )

    return router


async def healthcheck_endpoint(
    request: Request = None,
    app_name: str = "FastAPI Application",
    config_path: Optional[str] = None,
    environment: Optional[str] = None
):
    """
    Standalone FastAPI health check endpoint.

    Can be used directly as a route handler:
        @app.get("/healthcheck")
        async def health(request: Request):
            return await healthcheck_endpoint(request)
    """
    return await _healthcheck_handler(request, app_name, config_path, environment)


async def _healthcheck_handler(
    request: Optional[Request],
    app_name: str,
    config_path: Optional[str],
    environment: Optional[str]
):
    """Internal handler for health check logic."""

    # Load configuration and run checks
    if environment is None:
        environment = "development"

    load_config(config_path, environment)
    registry = get_registry()
    results = registry.run_all(environment)

    # Calculate statistics and overall status
    stats = _calculate_stats(results)
    overall_status = _get_overall_status(stats)

    # Determine response format
    wants_json = False
    if request:
        accept_header = request.headers.get("accept", "")
        format_param = request.query_params.get("format")
        wants_json = (
            "application/json" in accept_header or
            format_param == "json" or
            request.url.path.endswith(".json")
        )

    # Determine HTTP status code
    status_code = 200 if overall_status == "passed" else 503

    if wants_json:
        # Return JSON response
        response_data = _format_json_response(
            results, stats, overall_status, app_name, environment
        )
        return JSONResponse(content=response_data, status_code=status_code)
    else:
        # Return HTML response
        context = {
            'results': results,
            'stats': stats,
            'overall_status': overall_status,
            'app_name': app_name,
            'environment': environment,
            'timestamp': datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        }

        html_content = _render_html_template(context)
        return HTMLResponse(content=html_content, status_code=status_code)


def _calculate_stats(results):
    """Calculate statistics from check results."""
    stats = {
        "total": len(results),
        "passed": 0,
        "failed": 0,
        "skipped": 0,
        "error": 0
    }

    for _, result in results:
        if result.status == CheckStatus.PASSED:
            stats["passed"] += 1
        elif result.status == CheckStatus.FAILED:
            stats["failed"] += 1
        elif result.status == CheckStatus.SKIPPED:
            stats["skipped"] += 1
        elif result.status == CheckStatus.ERROR:
            stats["error"] += 1

    # Combine failed and error for simpler display
    stats["failed"] += stats["error"]

    return stats


def _get_overall_status(stats):
    """Determine overall health status."""
    if stats["failed"] > 0:
        return "failed"
    elif stats["total"] == stats["skipped"]:
        return "no_checks"
    elif stats["passed"] > 0:
        return "passed"
    else:
        return "unknown"


def _format_json_response(results, stats, overall_status, app_name, environment):
    """Format results for JSON response."""
    json_results = []
    for check, result in results:
        json_results.append({
            "description": check.description,
            "status": result.status.value,
            "message": result.message,
            "error": result.error,
            "duration_ms": result.duration_ms,
            "skip_reason": result.skip_reason,
        })

    return {
        "status": overall_status,
        "stats": stats,
        "environment": environment,
        "app_name": app_name,
        "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        "checks": json_results,
    }


def _render_html_template(context):
    """Render HTML template using Jinja2."""
    # Use the same template as Flask but render with Jinja2 directly
    template_str = _get_html_template()
    template = Template(template_str)
    return template.render(**context)


def _get_html_template():
    """Get HTML template string."""
    # For brevity, returning a simple template
    # In a real implementation, this would be the full template
    return """
<!DOCTYPE html>
<html>
<head>
    <title>Health Check - {{ app_name }}</title>
    <style>
        body { font-family: sans-serif; margin: 40px; }
        .header { text-align: center; margin-bottom: 2rem; }
        .pass { color: green; }
        .fail { color: red; }
        .skip { color: orange; }
        .summary { display: flex; gap: 2rem; justify-content: center; margin: 2rem 0; }
        .summary-item { text-align: center; padding: 1rem; background: #f5f5f5; border-radius: 8px; }
    </style>
</head>
<body>
    <div class="header">
        <h1>Health Check: {{ app_name }}</h1>
        <p>Status: <strong>{{ overall_status }}</strong></p>
        <p>Environment: {{ environment }} | {{ timestamp }}</p>
    </div>

    <div class="summary">
        <div class="summary-item">
            <div style="font-size: 2rem; color: green;">{{ stats.passed }}</div>
            <div>Passed</div>
        </div>
        <div class="summary-item">
            <div style="font-size: 2rem; color: red;">{{ stats.failed }}</div>
            <div>Failed</div>
        </div>
        <div class="summary-item">
            <div style="font-size: 2rem; color: orange;">{{ stats.skipped }}</div>
            <div>Skipped</div>
        </div>
    </div>

    <h2>Detailed Results</h2>
    <ul>
    {% for check, result in results %}
        <li class="{{ result.status.value }}">
            <strong>{{ check.description }}</strong>: {{ result.status.value }}
            {% if result.duration_ms %} ({{ "%.1f"|format(result.duration_ms) }}ms){% endif %}
            {% if result.skip_reason %}<br><em>{{ result.skip_reason }}</em>{% endif %}
            {% if result.message and result.status.value != "passed" %}<br><em>{{ result.message }}</em>{% endif %}
        </li>
    {% endfor %}
    </ul>
</body>
</html>
"""



import time

import pytest

from allgreen import (
    CheckStatus,
    check,
    get_registry,
    make_sure,
)


class TestTimeoutFunctionality:
    def test_check_with_timeout_passes(self):
        """Test that checks completing within timeout work normally."""
        registry = get_registry()
        registry.clear()

        @check("Quick check with timeout", timeout=2)
        def quick_check():
            time.sleep(0.1)  # Well under timeout
            make_sure(True)

        check_obj = registry.get_checks()[0]
        result = check_obj.execute()

        assert result.status == CheckStatus.PASSED
        assert result.duration_ms is not None
        assert result.duration_ms > 100  # At least 100ms due to sleep

    def test_check_timeout_error(self):
        """Test that checks exceeding timeout are handled properly."""
        registry = get_registry()
        registry.clear()

        @check("Slow check with timeout", timeout=1)
        def slow_check():
            time.sleep(2)  # Exceeds 1 second timeout
            make_sure(True, "Should not reach this")

        check_obj = registry.get_checks()[0]
        result = check_obj.execute()

        assert result.status == CheckStatus.ERROR
        assert "timed out after 1 seconds" in result.error
        assert result.message == "Check timed out"
        assert result.duration_ms is not None

    def test_default_timeout(self):
        """Test that checks use default 10 second timeout."""
        registry = get_registry()
        registry.clear()

        @check("Check with default timeout")
        def default_timeout_check():
            # This should complete well within 10 seconds
            make_sure(True)

        check_obj = registry.get_checks()[0]
        assert check_obj.timeout == 10

    def test_custom_timeout(self):
        """Test that custom timeouts are properly set."""
        registry = get_registry()
        registry.clear()

        @check("Check with custom timeout", timeout=30)
        def custom_timeout_check():
            make_sure(True)

        check_obj = registry.get_checks()[0]
        assert check_obj.timeout == 30

    def test_timeout_with_rate_limiting(self):
        """Test timeout works with rate-limited checks."""
        registry = get_registry()
        registry.clear()

        @check("Rate limited timeout check", timeout=1, run="10 times per hour")
        def rate_limited_timeout_check():
            time.sleep(2)  # Will timeout
            make_sure(True)

        check_obj = registry.get_checks()[0]

        # First few runs should timeout normally (we have plenty of quota)
        result = check_obj.execute()
        assert result.status == CheckStatus.ERROR
        assert "timed out" in result.error

        # Verify rate limiting is configured
        assert check_obj.run == "10 times per hour"

    def test_timeout_with_assertion_error(self):
        """Test that assertion errors within timeout work properly."""
        registry = get_registry()
        registry.clear()

        @check("Failing check with timeout", timeout=5)
        def failing_check_with_timeout():
            time.sleep(0.1)  # Short delay, well under timeout
            make_sure(False, "This check should fail")

        check_obj = registry.get_checks()[0]
        result = check_obj.execute()

        assert result.status == CheckStatus.FAILED
        assert "This check should fail" in result.message
        assert result.duration_ms is not None
        assert result.duration_ms > 100  # Should include sleep time

    def test_timeout_with_exception(self):
        """Test that regular exceptions within timeout are handled."""
        registry = get_registry()
        registry.clear()

        @check("Exception check with timeout", timeout=5)
        def exception_check_with_timeout():
            time.sleep(0.1)
            raise ValueError("Something went wrong")

        check_obj = registry.get_checks()[0]
        result = check_obj.execute()

        assert result.status == CheckStatus.ERROR
        assert "ValueError: Something went wrong" in result.error
        assert "Something went wrong" in result.message  # Should have traceback

    @pytest.mark.skipif(
        not hasattr(time, 'sleep'),
        reason="Platform doesn't support sleep"
    )
    def test_very_short_timeout(self):
        """Test very short timeouts work correctly."""
        registry = get_registry()
        registry.clear()

        @check("Very short timeout", timeout=1)
        def very_short_timeout():
            time.sleep(1.5)  # Slightly over 1 second
            make_sure(True)

        check_obj = registry.get_checks()[0]
        start_time = time.time()
        result = check_obj.execute()
        end_time = time.time()

        # Should timeout quickly
        assert result.status == CheckStatus.ERROR
        assert "timed out" in result.error
        # Should complete in roughly the timeout period (plus some overhead)
        assert (end_time - start_time) < 2.0  # Should be close to 1 second

    def test_zero_timeout_handling(self):
        """Test that zero timeout is handled sensibly."""
        registry = get_registry()
        registry.clear()

        # Zero timeout should default to something reasonable
        @check("Zero timeout check", timeout=0)
        def zero_timeout_check():
            make_sure(True)

        check_obj = registry.get_checks()[0]
        # Zero timeout should be converted to default
        assert check_obj.timeout == 10  # Default timeout
